#!/usr/bin/perl -w
# Ensembl の pep データベースから最も長いアミノ酸配列だけを取り出す．
#   それぞれの ensg には複数の ensp (トランスクリプト) があることが多いので，
#   系統解析などには最長の ensp 配列だけを用いたいことが多いです．

# perl longPepPicker.pl DB.pep.fa

use strict;
use warnings;

my $infile = "$ARGV[0]";
#my $outFile = "OnlyLongestPeps.txt";
my $outFile = "OnlyLongestPeps_".$infile;

# db の情報をハッシュや配列に格納するfasHashReaderWhile にインファイル名を渡す．
my ($ref_fasHash,$ref_nameLineArray,$ref_ensgs,$ref_ensps,$ref_ensts) = &fasHashReaderWhile($infile);
my %fasHash = %$ref_fasHash;
my @nameLineArray = @$ref_nameLineArray;
my @ensgs = @$ref_ensgs;
my @ensps = @$ref_ensps;
my @ensts = @$ref_ensts;


### プリントアウト
print "File name: $infile\n";
print " number of records: ".@nameLineArray."\n";
print " number of ensg ids: ".@ensgs."\n";
print " number of ensp ids: ".@ensps."\n";
#  foreach (@ensps) {
#    print "  $_\n";
#  }
print " number of enst ids: ".@ensts."\n";


#### スクリーンアウトだけ得たい場合は，下の行をアクティベートしてください．
#　　　スクリーンアウトだけなら，巨大ファイルでもそれほど時間はかかりません．
#exit;

# それぞれの ensg から最長の配列をとってきて，アウトファイルにプリントアウトする．
open(OUT,">$outFile");
foreach my $ensg (@ensgs) {
  my @enspGreped = grep/$ensg/,@nameLineArray;        # 同じ ensg ID のレコードを集める
  my @length = ( );
  foreach (@enspGreped) {
    my $seq = $fasHash{$_};                      
    push(@length,[$_,length($seq),$seq]);              # [レコード名，配列長，配列]という無名配列を作る
  }
  my @length2 = sort {$b->[1] <=> $a->[1]} @length;    # 上の無名配列を配列長でソートする
  my $ref_lengestRecord = shift@length2;               # 最長の配列だけをとる
  
  print OUT "$ref_lengestRecord->[0]\n$ref_lengestRecord->[2]\n";
}
close OUT;



exit;


#######  サブルーチン
# インファイルをあける．
sub fasHashReaderWhile {
  my $nameLine = "";
  my $seq = "";
  my %fasHash = ( );
  my @ensgs = ( );
  my @ensps = ( );
  my @ensts = ( );
  my @nameLineArray = ( );
  my $flag = 0;

  open(INFILE,"$infile") or die "$!";  
  while(my $inFileLine = <INFILE>) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){
      if($flag == 0){
        $nameLine = $inFileLine;
        $flag = 1;  
      } else {

        $nameLine =~ />(\w+) .*?gene:(\w+).*?transcript:(\w+)$/;
        my $ensp = $1;
        my $ensg = $2;
        my $enst = $3;
        my $seqLength = length($seq);
        push(@ensgs,$ensg);
        push(@ensps,$ensp);
        push(@ensts,$enst);
        push(@nameLineArray,$nameLine);
        $fasHash{$nameLine} = $seq;
        
        $nameLine = "$inFileLine";
        $seq = "";
      }
    } else {
      $seq .= $inFileLine; 
    }
  }

  $nameLine =~ />(\w+) .*?gene:(\w+).*?transcript:(\w+)$/;
  my $ensp = $1;
  my $ensg = $2;
  my $enst = $3;
  my $seqLength = length($seq);
  push(@ensgs,$ensg);
  push(@ensps,$ensp);
  push(@ensts,$enst);
  push(@nameLineArray,$nameLine);
  $fasHash{$nameLine} = $seq;
  
  close INFILE;
  
  ##-- 重複削除と sort --#
  my @ensgsUniq = sort(uniqArray(\@ensgs));
  my @enspsUniq = sort(uniqArray(\@ensps));
  my @enstsUniq = sort(uniqArray(\@ensts));

  return(\%fasHash,\@nameLineArray,\@ensgsUniq,\@enspsUniq,\@enstsUniq);
}

### 配列から重複を取り除く
sub uniqArray{
    my $array = shift;
    my %hash  = ();

    foreach my $value ( @$array ){
        $hash{$value} = 1;
    }

    return(
        keys %hash
    );
}

